import 'dart:math';

import 'package:flutter/material.dart';
import 'package:flutter/rendering.dart';

//TODO spin ve spinu držet si úhel ve struktuře dítě - rodič

class WireframeSingleWidgetTestScreen extends StatefulWidget {
  const WireframeSingleWidgetTestScreen({Key? key}) : super(key: key);

  @override
  State<WireframeSingleWidgetTestScreen> createState() => _WireframeSingleWidgetTestScreenState();
}

class _WireframeSingleWidgetTestScreenState extends State<WireframeSingleWidgetTestScreen> with TickerProviderStateMixin {
  bool switchValue = false;
  late final AnimationController _controller;
  late final AnimationController _sizeController;
  late final Animation<double> _sizeAnimation;
  late final Animation<double> _positionAnimation;

  @override
  void initState() {
    super.initState();
    _controller = AnimationController(
      duration: const Duration(seconds: 5),
      vsync: this,
    )..repeat();

    _sizeController = AnimationController(
      duration: const Duration(seconds: 5),
      vsync: this,
    )..repeat(reverse: true);

    _sizeAnimation = Tween<double>(begin: 50.0, end: 150.0).animate(_controller);
    _positionAnimation = Tween<double>(begin: 0.0, end: 1.0).animate(_controller);
  }

  @override
  void dispose() {
    _controller.dispose();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      body: Center(
        child: ListView(
          children: [
            /*
            const TextField(
              decoration: InputDecoration(
                filled: true,
                border: OutlineInputBorder(borderSide: BorderSide(width: 30)),
                labelText: "TextField",
                labelStyle: TextStyle(
                  fontSize: 12.0,
                  fontWeight: FontWeight.w400,
                ),
                alignLabelWithHint: true,
              ),
            ),
            Center(
              child: Container(
                width: 200.0, // Size of the circle
                height: 200.0, // Size of the circle
                child: AnimatedBuilder(
                  animation: _controller,
                  child: Container(
                    width: 50.0, // Width of the red square
                    height: 50.0, // Height of the red square
                    color: Colors.red,
                  ),
                  builder: (context, child) {
                    return Transform.translate(
                      offset: Offset(75, 0).rotate(_controller.value * 2 * pi),
                      child: child,
                    );
                  },
                ),
              ),
            ),*/
            SizedBox(height: 100),
            AnimatedBuilder(
              animation: _controller,
              child: Container(
                width: 50.0,
                height: 50.0,
                color: Colors.green,
              ),
              builder: (context, child) {
                return Transform.rotate(
                  angle: _controller.value * 2 * pi,
                  child: child,
                );
              },
            ),
            UnconstrainedBox(
              child: AnimatedBuilder(
                animation: _controller,
                child: Container(
                  width: 50.0,
                  height: 50.0,
                  color: Colors.blue,
                ),
                builder: (context, child) {
                  return Transform.rotate(
                    angle: _controller.value * 2 * pi,
                    child: child,
                  );
                },
              ),
            ),
            Transform.rotate(
              child: Container(
                width: 50.0,
                height: 50.0,
                color: Colors.red,
              ),
              angle: 0.9 * pi,
            ),

            /*
            SizedBox(height: 100),
            AnimatedBuilder(
              animation: _sizeController,
              builder: (context, child) {
                return Align(
                  alignment: Alignment(_positionAnimation.value * 2 - 1, 0),
                  child: Container(
                    width: _sizeAnimation.value,
                    height: _sizeAnimation.value,
                    color: Colors.blue,
                  ),
                );
              },
            ),*/
          ],
        ),
      ),
    );
  }

  Future<void> _showMyDialog(BuildContext context) async {
    return showDialog<void>(
      context: context,
      barrierDismissible: false, // user must tap button!
      builder: (BuildContext context) {
        return const TestDialog();
      },
    );
  }
}

extension on Offset {
  Offset rotate(double angle) {
    return Offset(
      cos(angle) * dx - sin(angle) * dy,
      sin(angle) * dx + cos(angle) * dy,
    );
  }
}

class TestDialog extends StatelessWidget {
  const TestDialog({Key? key}) : super(key: key);

  @override
  Widget build(BuildContext context) {
    return AlertDialog(
      shape: const RoundedRectangleBorder(
        borderRadius: BorderRadius.all(
          Radius.circular(6.0),
        ),
      ),
      insetPadding: const EdgeInsets.symmetric(horizontal: 100),
      title: const Text(
        'Premium Account',
        style: TextStyle(
          fontSize: 22.0,
          color: Colors.black,
          fontWeight: FontWeight.w500,
        ),
      ),
      content: SingleChildScrollView(
        child: ListBody(
          children: <Widget>[
            const Text(
              'In order to download this movie, you need to have premium account.',
              style: TextStyle(
                fontSize: 16.0,
                color: Colors.black,
                fontWeight: FontWeight.w400,
              ),
            ),
            const SizedBox(height: 20),
            const Text(
              'Would you like to purchase it?',
              style: TextStyle(
                fontSize: 16.0,
                color: Colors.black,
                fontWeight: FontWeight.w400,
              ),
            ),
          ],
        ),
      ),
      actions: <Widget>[
        TextButton(
          child: const Text(
            'CANCEL',
            style: TextStyle(
              fontSize: 18.0,
              color: Colors.black,
              fontWeight: FontWeight.w700,
            ),
          ),
          onPressed: () {
            Navigator.of(context).pop();
          },
        ),
        TextButton(
          child: const Text(
            'PURCHASE',
            style: TextStyle(
              fontSize: 18.0,
              color: Colors.blue,
              fontWeight: FontWeight.w700,
            ),
          ),
          onPressed: () {},
        ),
      ],
    );
  }
}

class Sky extends CustomPainter {
  @override
  void paint(Canvas canvas, Size size) {
    final Rect rect = Offset.zero & size;

    const RadialGradient gradient = RadialGradient(
      center: Alignment(0.7, -0.6),
      radius: 0.2,
      colors: <Color>[Color(0xFFFFFF00), Color(0xFF0099FF)],
      stops: <double>[0.4, 1.0],
    );
    canvas.drawRect(
      rect,
      Paint()..shader = gradient.createShader(rect),
    );
  }

  @override
  SemanticsBuilderCallback get semanticsBuilder {
    return (Size size) {
      Rect rect = Offset.zero & size;
      final double width = size.shortestSide * 0.4;
      rect = const Alignment(0.8, -0.9).inscribe(Size(width, width), rect);

      return <CustomPainterSemantics>[
        CustomPainterSemantics(
          rect: rect,
          properties: const SemanticsProperties(
            label: 'Sun',
            textDirection: TextDirection.ltr,
          ),
        ),
      ];
    };
  }

  @override
  bool shouldRepaint(Sky oldDelegate) => false;

  @override
  bool shouldRebuildSemantics(Sky oldDelegate) => false;
}
